define([
		"dojo/_base/declare",
		"aps/Grid",
		"aps/DateTextBox",
		"dojo/dom-construct",
		"dojo/_base/array",
		"dojo/query"
], function(declare, Grid, DateTextBox, domConstruct, arr, query){
	return declare([Grid], {
		_apsUiRuntime21: false,
		customDateFilterData: null,

		postMixInProperties: function(){
			var self = this;
			self.inherited(arguments);
			self._apsUiRuntime21 = false;
		},

		_setCustomDateFilterDataAttr: function(val){
			var self = this;
			self._set("customDateFilterData", val);
		},

		_getApsUiRuntime21Attr: function(){
			var self = this;
			return self._apsUiRuntime21;
		},

		_createSearchPanel: function(root){
			var self = this,
				ul,
				node,
				frag = document.createDocumentFragment(),
				searchBoxes	 = query(".search-box", root);

			self._apsUiRuntime21 = true;

			if(searchBoxes.length > 0)
				arr.forEach(searchBoxes, function(box){ domConstruct.destroy(box); });

			function submit(){
				self.set("_filters", []);
				query("input, select", self.toolbarNode).forEach(function(item){
					if(item.id && item.value)
						self._filters[item.id.split('-').pop()] = item.value;
				});
				self.selectionArray.splice(0, self.selectionArray.length);
				self._query(0, null, null, self.get("rowsPerPage"), 1);
			}

			function checkCustomDateFilter(filter){
				return filter && filter.name && filter.start && filter.end;
			}

			function prepareFilter(caption){
				if(!node)
					node = query("form", domConstruct.create("div", { "class": "search-box", innerHTML: "<form></form>" }, frag))[0];

				if(!ul)
					ul = domConstruct.create("ul", {}, node);

				var li = domConstruct.create("li", { innerHTML: "<div><span>" + caption + "</span><span></span></div>" }, ul);

				return query("span", li)[1];
			}

			if (checkCustomDateFilter(self.customDateFilterData)) {
				var startData = self.customDateFilterData.start;
				var li_start_body = prepareFilter(startData.title);

				var start = new DateTextBox({
					id: self.id + "-search-date-start-" + self.customDateFilterData.name,
					value: startData.value,
					autoSize: true,
					missingMessage: startData.missingMessage,
					required: true,
					constraints: startData.constraints,
					setDefault: startData.setDefault,
					setNewValue: startData.setNewValue,
					validateAndShowError: startData.validateAndShowError
				});

				var endData = self.customDateFilterData.end;
				var end = new DateTextBox({
					id: self.id + "-search-date-end-" + self.customDateFilterData.name,
					value: endData.value,
					autoSize: true,
					missingMessage: endData.missingMessage,
					required: true,
					constraints: endData.constraints,
					setDefault: endData.setDefault,
					setNewValue: endData.setNewValue,
					validateAndShowError: endData.validateAndShowError
				});

				start.own(start.watch("value", function(name, oldVal, newVal){
					if (newVal){
						var oldContrains = end.get("constraints");
						end.set("constraints", { min: newVal, max: oldContrains.max });
						start.setNewValue(newVal);
					}
				}));
				end.watch(end.watch("value", function(name, oldVal, newVal){
					if (newVal){
						var oldContrains = start.get("constraints");
						start.set("constraints", { max: newVal, min: oldContrains.min });
						end.setNewValue(newVal);
					}
				}));

				domConstruct.place(start.domNode, li_start_body);

				var li_end = domConstruct.create("li", { innerHTML: "<div><span>" + endData.title + "</span><span></span></div>" }, ul);
				var li_end_body = query("span", li_end)[1];
				domConstruct.place(end.domNode, li_end_body);
			}

			arr.forEach(this.columns, function(column){
				var type = column.type ? column.type.toLowerCase() : false;
				if (type && (type === "date" || type === "datetime")) {
					return;
				}
				if(column.filter){
					var li_body = prepareFilter(column.filter.title ? column.filter.title : column.name);
					if(column.filter.options){
						var select = domConstruct.create("select", { id: self.id + "-search-select-" + column.field	}, li_body);
						domConstruct.create("option", {
							innerHTML: self.filterOptAllText,
							selected: "selected",
							value: ""
						}, select);
						arr.forEach(column.filter.options, function(option){
							var opt = domConstruct.create("option", {
								innerHTML: option.label,
								value: typeof option.value === "boolean" ? option.value + "()" : option.value
							}, select);

							if(opt.value == column.filter.value)
								opt.selected = "selected";
						});
					} else {
						domConstruct.create("input", {
							id: self.id + "-search-text-" + column.field,
							type: "text",
							value: column.filter.value || ""
						}, li_body);
					}
				}
			});

			if(node){
				query("input, select", node).on("keypress", function(e){
					if(e.keyCode == 13){
						submit();
						e.preventDefault();
					}
				});

				domConstruct.create("a", {
					"class": "s-btn sb-search",
					href: "javascript:void(0)",
					innerHTML: "<i><span>" + self.searchButtonText + "</span></i>",
					onclick: submit
				}, node);

				domConstruct.create("a", {
					"class": "s-btn sb-show-all",
					href: "javascript:void(0)",
					innerHTML: "<i><span>" + self.resetSearchButtonText + "</span></i>",
					onclick: function(){
						self.set({ "_filters": [] });
						self.selectionArray.splice(0, self.selectionArray.length);
						query("input, select", self.toolbarNode).forEach(function(item){
							item.value = "";
							if(item.selectedIndex) item.selectedIndex = 0;
						});
						self._query(0, null, null, self.get("rowsPerPage"));
					}
				}, node);
			}
			domConstruct.place(frag, root);
		}
	});
});